#include "3dstypes.h"
#include "draw.h"
#include "libc.h"
#include "utils.h"
#include "hid.h"
#include "FS.h"
#include "i2c.h"
#include "config.h"

static void reboot();
static void setOptions();
static void loadSplash();

#define BOOTIMAGE_ADDR     (0x01ff8000)
#define BOOTIMAGE_FILENAME "/TRICK/boot.bin"   
#define CONFIG_FILENAME    "/TRICK/trick.cfg"   

char splash_file[] = "/TRICK/splash0.bin";
int splash_index;
int default_boot_option;
int delay;
int second_emunand;
u32 rednand_start_sect; 

int main()
{
	unsigned int n = 0, bin_size;

	set_VRAM_framebuffers(0);

	ClearScreen(SCREEN_TOP | SCREEN_BOT);
	
	Handle fd;
 
	fd = FileOpen(CONFIG_FILENAME, OPEN_READ);
	if (fd) {

	  u32 config_params[2] ;
	  FileRead(fd, config_params, 8, 0);
	  splash_index = config_params[0] & 0b11;
	  default_boot_option = (config_params[0] & 0b1100)>>2;
	  delay = (config_params[0] & 0b1110000)>>4;
	  second_emunand = (config_params[0] & 0b10000000)>>7;
      rednand_start_sect = config_params[1]; 
	} else {
	  splash_index = 0;
	  default_boot_option = 1;
	  delay = 6;
	  second_emunand = 1;
	  rednand_start_sect = 0x00200001; 
	  u32 config_params[2] ;
	  fd = FileOpen(CONFIG_FILENAME, OPEN_CREATE|OPEN_WRITE);
      config_params[0] = splash_index | (default_boot_option<<2) | (delay<<4) | (second_emunand<<7);
      config_params[1] = rednand_start_sect; 
	  FileWrite(fd, config_params, 8, 0);
	}

	loadSplash();	
	
	DBGCOL(SCREEN_BOT, CYAN, "TRICK - 3DS CFW\n\n");

	fd = FileOpen(BOOTIMAGE_FILENAME, OPEN_READ);
	
	if (fd==0) {
		ERROR(SCREEN_BOT,
			"Boot.bin not found\n"
			"Press START to reboot.");
	}
	
	//Load the boot code
	bin_size = 0;
	while ((n = FileRead(fd, (void*)((u32)BOOTIMAGE_ADDR+bin_size), 0x100000, bin_size)) > 0) {
		bin_size += n;
	}
	FlushProcessDataCache(CURRENT_KPROCESS, (void *)BOOTIMAGE_ADDR, bin_size);
	DEBUG(SCREEN_BOT, "Boot.bin loaded.\n");

	u32 selected=default_boot_option;
	u64 start;
	start= getSystemTick();
	int iSecs = delay;
	
	DBGCOL(SCREEN_BOT, CYAN, "\nPress L or R to show booting option.\n");
	u32 console_y=console_get_y();
	
	do {
	  if(BUTTON_PRESS((BUTTON_L1 | BUTTON_R1))) selected=0;
	  if(getSystemTick()-start>(s32)((7-iSecs)*TICKS_PER_SEC)) {
	    console_set_y(console_y);
	    DBGCOL(SCREEN_BOT, YELLOW, "Booting default in %i seconds\n",--iSecs);
	  }
	} while((iSecs >0)&&(selected));

	if (!selected) {
	  DEBUG(SCREEN_BOT, "\nSelect a Boot option:\n");
	  DBGCOL(SCREEN_BOT, GREEN, "  (A) Boot emunand\n");
	  DBGCOL(SCREEN_BOT, GREEN, "  (B) Boot sysnand (no sign check)\n");
	  if(second_emunand){
	    DBGCOL(SCREEN_BOT, GREEN, "  (X) Boot second emunand\n");
	  }
	  DBGCOL(SCREEN_BOT, GREEN, "  (SELECT) Config options\n");
	  DBGCOL(SCREEN_BOT, GREEN, "  (START) Back to home menu\n");

	  while (!selected) {
	    if(BUTTON_PRESS(BUTTON_A)) selected=1;
	    if(BUTTON_PRESS(BUTTON_B)) selected=2;
	    if(second_emunand)
	      if(BUTTON_PRESS(BUTTON_X)) selected=3;	  
	    if(BUTTON_PRESS(BUTTON_SELECT)) setOptions();	  
	    if(BUTTON_PRESS(BUTTON_START)) reboot();	  
	  }
	}

	switch (selected) {
	case 1: 
	  DBGCOL(SCREEN_BOT, YELLOW, "\nBooting emunand ... ");
	  break;
	case 2: 
	  DBGCOL(SCREEN_BOT, RED, "\nWARNING: you selected to boot sysnand with code sign check removed.\n");
	  DBGCOL(SCREEN_BOT, RED, "Installing system content in this modecan brick your 3ds!\n");
	  DBGCOL(SCREEN_BOT, GREEN, "\nSelect: (A) Confirm (X) Back to home\n");
	  u32 confirm=0;
	  while (!confirm) {
	    if(BUTTON_PRESS(BUTTON_A)) confirm=1;
	    if(BUTTON_PRESS(BUTTON_X)) reboot();	  
	  }
	  DBGCOL(SCREEN_BOT, YELLOW, "\nBooting sysnand ... ");
	  *(u32*)(BOOTIMAGE_ADDR+0x20) = 0xE1A00000; // NOP the branch to the fuction that makes the first part of the SD redirection patching
	  for(n=0x1958;n<0x1988;n+=4)
	    *(u32*)(BOOTIMAGE_ADDR+n) = 0xE1A00000; // NOP the 12 arm instructions that complete the SD redirection patching
	  break;
	case 3: 
	  DBGCOL(SCREEN_BOT, YELLOW, "\nBooting second emunand ... ");
	  ((u32*)BOOTIMAGE_ADDR)[5]= rednand_start_sect; 
	  break;
	}


	return 0;
}


void reboot()
{
    i2cWriteRegister(I2C_DEV_MCU, 0x20, 1 << 2);
    while(true);
}

void setOptions()
{
	const char* Labels[6]= {"None","Emunand","Sysnand","Second emunand","Inactive","Active"};
	const char Hex_Ciphers[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
	int selection = 0;
	int pos = 0;
	int i,temp;
	u32 key;
	while(1) {
	  ClearScreen(SCREEN_BOT);
	  console_reset();
	  DBGCOL(SCREEN_BOT, CYAN, "TRICK - 3DS CFW\n\n");
	  DBGCOL(SCREEN_BOT, CYAN, "Options configuration.\n\n");
	  DEBUG(SCREEN_BOT, "Top image     : ");
	  DBGCOL(SCREEN_BOT, (selection==0)?RED:WHITE, "%i",splash_index);
	  DEBUG(SCREEN_BOT, "\nDefault boot  : ");
	  DBGCOL(SCREEN_BOT, (selection==1)?RED:WHITE, Labels[default_boot_option]);
	  DEBUG(SCREEN_BOT, "\nBoot delay    : ");
	  DBGCOL(SCREEN_BOT, (selection==2)?RED:WHITE, "%i",delay);
	  DEBUG(SCREEN_BOT, "\nSecond emunand: ");
	  DBGCOL(SCREEN_BOT, (selection==3)?RED:WHITE, Labels[4+second_emunand]);
	  if(second_emunand){
	    DEBUG(SCREEN_BOT, "\n  start sector: ");
        for (i=0;i<8;i++) {
          temp = (rednand_start_sect>>((7-i)*4)) & 0b1111;
		  DBGCOL(SCREEN_BOT, ((selection==4)&&(pos==i))?RED:WHITE, "%c ", Hex_Ciphers[temp]);
	    }
	  }
	  DBGCOL(SCREEN_BOT,YELLOW, "\n\nUse arrows to select/change options.\n");
	  if (selection == 4)
		DBGCOL(SCREEN_BOT,YELLOW, "Press B to increase selected value.\n");
	  DBGCOL(SCREEN_BOT,YELLOW, "Press A to save.");

	  key = InputWait();
      if (key & BUTTON_UP)	{
	    selection--;
	    if (selection < 0) selection = 4;
	    if ((selection == 4)&&(!second_emunand)) selection = 3;
	  }
      if (key & BUTTON_DOWN) {
	    selection++;
	    if (selection > 4) selection=0;
	    if ((selection == 4)&&(!second_emunand)) selection = 0;
	  }
      if (key & BUTTON_RIGHT) {
	    if (selection == 0) {
		  splash_index = (splash_index+1) & 0b11;
		  loadSplash();
	    }
	    if (selection == 1) {
		  default_boot_option = (default_boot_option+1) & 0b11;
		  if ((default_boot_option==3)&&!second_emunand) default_boot_option = 0;
	    }
		if (selection == 2) delay = (delay+1) & 0b111;
	    if (selection == 3) second_emunand = (second_emunand+1) & 0b1;
	    if (selection == 4) pos = (pos+1) & 0b111;
	  }
      if (key & BUTTON_LEFT) {
	    if (selection == 0) {
		  splash_index = (splash_index-1) & 0b11;
		  loadSplash();
	    }
	    if (selection == 1) {
		  default_boot_option = (default_boot_option-1) & 0b11;
		  if ((default_boot_option==3)&&!second_emunand) default_boot_option = 2;
	    }
	    if (selection == 2) delay = (delay-1) & 0b111;
	    if (selection == 3) second_emunand = (second_emunand+1) & 0b1;
	    if (selection == 4) pos = (pos-1) & 0b111;
	  }
      if ((second_emunand==0)&&(default_boot_option == 3)) default_boot_option = 0;
      if ((key & BUTTON_B)&&(selection == 4)) {
	    temp = ((rednand_start_sect>>((7-pos)*4))+1) & 0b1111;
		rednand_start_sect = (rednand_start_sect & (0xffffffff ^ (0b1111<< ((7-pos)*4)))) | (temp<< ((7-pos)*4));
	  }
      if (key & BUTTON_A) {
	    u32 config_params[2] ;
	    Handle fd = FileOpen(CONFIG_FILENAME,  OPEN_WRITE);
        config_params[0] = splash_index | (default_boot_option<<2) | (delay<<4) | (second_emunand<<7);
        config_params[1] = rednand_start_sect; 
	    FileWrite(fd, config_params, 8, 0);
		while(BUTTON_PRESS(BUTTON_A));
		break;
	  }
	}

	ClearScreen(SCREEN_BOT);
	console_reset();
	DBGCOL(SCREEN_BOT, CYAN, "TRICK - 3DS CFW\n\n");
	DEBUG(SCREEN_BOT, "\nSelect a Boot option:\n");
	DBGCOL(SCREEN_BOT, GREEN, "  (A) Boot emunand\n");
	DBGCOL(SCREEN_BOT, GREEN, "  (B) Boot sysnand (no sign check)\n");
	if(second_emunand){
	  DBGCOL(SCREEN_BOT, GREEN, "  (X) Boot second emunand\n");
	}
	DBGCOL(SCREEN_BOT, GREEN, "  (SELECT) Config options\n");
	DBGCOL(SCREEN_BOT, GREEN, "  (START) Back to home menu\n");
}

void loadSplash() {

	unsigned int n = 0, bin_size;

	splash_file[13] = '0' + splash_index;
	Handle fd = FileOpen(splash_file, OPEN_READ);
	if (fd) {
	  //Load the spash image
	  bin_size = 0;
	  while ((n = FileRead(fd, (void*)((u32)FB_TOP_LEFT1_FCRAM+bin_size), 0x100000, bin_size)) > 0) {
	    bin_size += n;
	  }
	  u32 *fb1 = (u32*) FB_TOP_LEFT1_FCRAM;
	  u32 *fb2 = (u32*) FB_TOP_LEFT2_FCRAM;
	  u32 *fb3 = (u32*) FB_TOP_RIGHT1_FCRAM;
	  u32 *fb4 = (u32*) FB_TOP_RIGHT2_FCRAM;
	  for(n=0;n<bin_size;n+=4){
	    *fb4++ = *fb3++ = *fb2++ = *fb1++;
	  }
	}
}
